/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.battery;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.SensorComponent;

/**
 * Provides access to the current state of the battery
 * 
 * @author Bjoern Richerzhagen
 * 
 */
public interface BatterySensor extends SensorComponent {

	/**
	 * Percentage (0..100) of battery level.
	 * 
	 * @return
	 */
	public double getCurrentPercentage();

	/**
	 * Check if battery is fully charged
	 * 
	 * @return true if fully charged, false otherwise
	 */
	public boolean isFullyCharged();

	/**
	 * Calculate the percentage amount of the given energy for the battery
	 * 
	 * @param energy
	 *            in uJ
	 * @return percentage
	 */
	public double energyPercentageOfBattery(double energy);

	/**
	 * Get the maxium capacity of this battery
	 * 
	 * @return
	 */
	public double energyCapacity();

	/**
	 * Returns true, if the battery is empty.
	 * 
	 * @return
	 */
	public boolean isEmpty();
}
