/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponentFactory;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.plugin.EnvironmentSensorPlugin;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 27.02.2018
 *
 */
public class ExtendableEnvironmentSensorFactory implements HostComponentFactory {
	private List<EnvironmentSensorPlugin> _plugins = new ArrayList<>();

	@Override
	public HostComponent createComponent(Host host) {
		List<EnvironmentSensorPlugin> plugins = new ArrayList<>();
		for (EnvironmentSensorPlugin environmentSensorPlugin : _plugins) {
			try {
				EnvironmentSensorPlugin clone = environmentSensorPlugin.clone();
				clone.setHost(host);
				plugins.add(clone);
			} catch (CloneNotSupportedException e) {
				throw new AssertionError("Each " + EnvironmentSensorPlugin.class.getSimpleName() + " must be clonable");
			}
		}
		return new ExtendableEnvironmentSensor(host, plugins);
	}

	public void setPlugin(EnvironmentSensorPlugin pPlugin) {
		_plugins.add(pPlugin);
	}
}
