/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 12.04.2018
 *
 */
public class TemporalDependencyMatrix implements Cloneable {
	// Value x | y --> Transition probability from state x to state y
	private double[][] _dependencies;

	private Map<Long, TemporalDependencyMatrix> _agedMatrices = new HashMap<>();

	public TemporalDependencyMatrix(int dimension) {
		_dependencies = new double[dimension][dimension];
	}

	private TemporalDependencyMatrix(double[][] pDependencies) {
		_dependencies = pDependencies;
	}

	public double[][] getDependencies() {
		return _dependencies;
	}

	public void setTransitionProbability(int pCurrent, double[] pProbabilities) {
		for (int y = 0; y < pProbabilities.length; y++) {
			_dependencies[pCurrent][y] = pProbabilities[y];
		}
	}

	public TemporalDependencyMatrix age(long pAge) {
		if (!_agedMatrices.containsKey(pAge)) {
			double[][] current = new double[_dependencies.length][];
			
			for (int i = 0; i < current.length; i++) {
				current[i] = new double[_dependencies[i].length];
				current[i][i] = 1;
			}
			
			for (long t = 0; t < pAge; t++) {
				double[][] next = createEmptyArray(current);
				
				for (int i = 0; i < current.length; i++) {
					for (int j = 0; j < current[i].length; j++) {
						for (int k = 0; k < current[i].length; k++) {
							next[i][j] += current[i][k] * _dependencies[k][j];
						}
					}
				}
				
				current = next;
			}
			
			_agedMatrices.put(pAge, new TemporalDependencyMatrix(current));
		}
		return _agedMatrices.get(pAge);
	}

	public double[][] cloneArray(double[][] pDependencies) {
		double[][] current = new double[pDependencies.length][];

		for (int i = 0; i < current.length; i++) {
			current[i] = Arrays.copyOf(pDependencies[i], pDependencies[i].length);
		}

		return current;
	}

	public double[][] createEmptyArray(double[][] pDependencies) {
		double[][] current = new double[pDependencies.length][];

		for (int i = 0; i < current.length; i++) {
			current[i] = new double[pDependencies[i].length];
		}

		return current;
	}

	public double getTransitionProbability(int pCurrent, int pNext) {
		return _dependencies[pCurrent][pNext];
	}

	@Override
	public TemporalDependencyMatrix clone() {
		double[][] dependencies = new double[_dependencies.length][];

		for (int i = 0; i < dependencies.length; i++) {
			dependencies[i] = Arrays.copyOf(_dependencies[i], _dependencies[i].length);
		}

		return new TemporalDependencyMatrix(dependencies);
	}

	/**
	 * @param pI
	 * @return
	 */
	public double getChangeProbability(int pI) {
		return 1 - _dependencies[pI][pI];
	}
}
