package de.tudarmstadt.maki.simonstrator.api.component.sensor.location;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.battery.RechargableBattery;
import de.tudarmstadt.maki.simonstrator.api.component.uavsupport.strategy.callbacks.ReachedLocationCallback;

/**
 * Interface for location actuators with energy consumption. Gives access to
 * actuator state control and sensors.
 * 
 * @author Julian Zobel
 *
 */
public interface EnergyConsumingLocationActuator extends LocationActuator {

	public boolean isAvailable();
	
	public boolean isActive();
	
	public boolean activate();
	
	public boolean deactivate();
	
	public boolean isMoving();
	
	public boolean setMoving();
	
	public boolean holdPosition();
	
	/**
	 * Energy calculation of this actuator based on a given distance. Calculates the
	 * energy needed in Joule when flying the given distance (with maximum available
	 * speed).
	 * 
	 * @param distance
	 *            Given distance that is used as calculation basis
	 * @return The energy in Joule, needed to fly the distance.
	 */
	public double energyCalculation(double distance, double speed);

	public double getChargePercent();

	public boolean turnOff();

	public double energyCalculationToLocation(Location location);

	public double calculateFlightDistanceToPoint(Location location);

	public boolean reachableDestination(Location location, double energylevelAtDestination);

	public void land();

	public void moveToLocation(Location location, ReachedLocationCallback cb);

	public void moveToLocation(Location location, double speed, ReachedLocationCallback cb);

	public void reachedLocation(Location location);

	public Location getCurrentLocation();

	public RechargableBattery getBattery();

}
